@ECHO OFF

ECHO Motorola Mobility flashall script version 1.4

:: always use fastboot that come with the package
SET fastboot=.\Windows\fastboot

::
:: command line options to override the defaults
:: useful for factory software preparation
::
SET flash_pds=0
SET flash_customize=0
SET flash_userdata=0
SET flash_cache=0
SET erase_userdata=1
SET erase_cache=1
SET leave_userdata=0
SET leave_cache=0

::
:: process command line options
::

:proc_option

IF "%1" == "" (
	GOTO :start_flashing
)

SET bad_option=1

:: flash customize?
IF "%1" == "/cz" (
        SET flash_customize=1
        SHIFT
        SET bad_option=0
)

:: flash pds?
IF "%1" == "/p" (
	SET flash_pds=1
	SHIFT
	SET bad_option=0
)

:: flash userdata and cache ?
IF "%1" == "/u" (
	SET flash_userdata=1
	SET flash_cache=1
	SHIFT
	SET bad_option=0
)

:: erase userdata?
IF "%1" == "/eu" (
	SET erase_userdata=1
	SHIFT
	SET bad_option=0
)

:: erase cache?
IF "%1" == "/ec" (
	SET erase_cache=1
	SHIFT
	SET bad_option=0
)

:: leave userdata?
IF "%1" == "/lu" (
	SET leave_userdata=1
	SET erase_userdata=0
	SHIFT
	SET bad_option=0
)

:: leave cache?
IF "%1" == "/lc" (
	SET leave_cache=1
	SET erase_cache=0
	SHIFT
	SET bad_option=0
)


IF %bad_option% == 1 (
	CALL :show_usage
	EXIT /b 1
)

GOTO :proc_option


::
:: Start flashing
::
:start_flashing

:: not to erase userdata if we are flashing it
:: because bootloader erases the partition before
:: flashing it
IF %flash_userdata% == 1 SET erase_userdata=0
IF %flash_cache% == 1 SET erase_cache=0

ECHO.
ECHO. Flashing Options:
ECHO. - flash customize: %flash_customize%
ECHO. - flash pds:      %flash_pds%
ECHO. - flash userdata: %flash_userdata%
ECHO. - flash cache : %flash_cache%
ECHO. - erase userdata: %erase_userdata%
ECHO. - erase cache:    %erase_cache%
ECHO. - leave userdata: %leave_userdata%
ECHO. - leave cache:    %leave_cache%
ECHO.

:: Wait for fastboot device enumeration
CALL :wait_for_device

REM CALL %fastboot% getvar product > product.txt 2>&1
REM FOR /F "tokens=2" %%a IN ('findstr /B "product:" product.txt') DO @set PRODUCT=%%a
REM DEL product.txt
REM IF "%PRODUCT%" == "#PRODNAME#" GOTO :validated_hardware
REM ECHO. Error: Cannot flash #PRODNAME# software on %PRODUCT% hardware
REM EXIT /b 1
REM :validated_hardware

ECHO.
ECHO. Flashing bootloader images...
ECHO.
CALL :fastboot_flash motoboot motoboot.bin
IF %errorlevel% NEQ 0 EXIT /b 1
CALL :fastboot_reboot_bl
IF %errorlevel% NEQ 0 EXIT /b 1
CALL :wait_for_device
GOTO :flash_ap

:flash_ap
ECHO.
ECHO. Flashing AP Images...
ECHO.

CALL :fastboot_flash boot boot.img
IF %errorlevel% NEQ 0 EXIT /b 1
CALL :fastboot_flash recovery recovery.img
IF %errorlevel% NEQ 0 EXIT /b 1
CALL :fastboot_multi_flash system system.img
IF %errorlevel% NEQ 0 EXIT /b 1

:: Flash customize?
IF %flash_customize% == 1 (
        GOTO :flash_customize
)
GOTO :check_and_flash_pds
:flash_customize
ECHO. Flashing customize...
CALL :fastboot_flash customize customize.img
:check_and_flash_pds
:: Flash pds?
IF %flash_pds% == 1 (
	GOTO :flash_pds
)
GOTO :erase_userdata
:flash_pds
ECHO. Flashing pds...
CALL :fastboot_flash pds pds.img
:: Erase userdata?
:erase_userdata
IF %erase_userdata% == 1 (
	GOTO :do_erase_userdata
)
GOTO :erase_cache
:do_erase_userdata
ECHO. Erasing userdata...
CALL :fastboot_erase userdata
IF %errorlevel% NEQ 0 EXIT /b 1

:: Erase cache?
:erase_cache
IF %erase_cache% == 1 (
	GOTO :do_erase_cache
)
GOTO :flash_userdata
:do_erase_cache
ECHO. Erasing cache
CALL :fastboot_erase cache
IF %errorlevel% NEQ 0 EXIT /b 1

:: Flash userdata?
:flash_userdata
IF %flash_userdata% == 1 (
	GOTO :do_flash_userdata
)
GOTO :flash_cache
:: Flash cache?
:flash_cache
IF %flash_cache% == 1 (
	GOTO :do_flash_cache
)
GOTO :all_done
:do_flash_userdata
ECHO. Flashing userdata...
CALL :fastboot_multi_flash userdata userdata.img
IF %errorlevel% NEQ 0 EXIT /b 1

:do_flash_cache
ECHO. Flashing Cache...
CALL :fastboot_multi_flash cache cache.img
IF %errorlevel% NEQ 0 EXIT /b 1

:all_done
ECHO.
ECHO. All images flashed successfully!
ECHO. Rebooting device...
ECHO.
IF %errorlevel% NEQ 0 EXIT /b 1
CALL %fastboot% oem fb_mode_clear
IF %errorlevel% NEQ 0 EXIT /b 1
%fastboot% reboot

PAUSE

EXIT /b 0

:: ------------------------------------------------------------------------
:: Functions start here
:: ------------------------------------------------------------------------

:nofastboot
	ECHO.
	ECHO. Could not find fastboot.exe in path
	ECHO.
	EXIT /b 0

:show_usage
	ECHO.
	ECHO. Flashes all images with fastboot
	ECHO.
	ECHO. flashall [/?] [/p] [/u] [/eu] [/ec] [/nm]
	ECHO.
	ECHO. userdata and cache are erased by default
	ECHO.
	ECHO. /?   Help
	ECHO. /cz  Flash customize
	ECHO. /p   Flash pds
	ECHO. /u   Flash userdata
	ECHO. /eu  Erase userdata
	ECHO. /ec  Erase cache
	ECHO. /lu  Leave userdata
	ECHO. /lc  Leave cache
	ECHO.
	EXIT /b 0

:wait_for_device
	SETLOCAL

	ECHO.
	ECHO. Waiting for fastboot enumeration...
	ECHO.

	SET /A wait_time=1

	:loop
	SET /A device_count=0

	FOR /F "usebackq tokens=2" %%X in (`%fastboot% devices`) do (
		IF "%%X" == "motorola" SET /A device_count+=1
		IF "%%X" == "fastboot" SET /A device_count+=1
	)

	IF %device_count% LSS 1 (
		ping -n 2 127.0.0.1 > NUL
		SET /A wait_time+=1
		IF %wait_time% == 10 (
			ECHO. Still waiting...
			ECHO. Unplug USB cable and reboot in fastboot mode.
			ECHO. Do you by any chance have a JTAG flex attached?
			ECHO. If so, detach the JTAG flex and try again.
			ECHO.

			SET /A wait_time=1
		)
		GOTO :loop
	)

	IF %device_count% GTR 1 (
		ECHO. Too many devices!
		ping -n 2 127.0.0.1 > NUL
		GOTO :loop
	)

	:: Allow time for fastboot enumeration to stablize
	ping -n 3 127.0.0.1 > NUL

	ECHO. Fastboot device is ready!!

	ENDLOCAL
	EXIT /b 0

:fastboot_flash
	ECHO. Executing "%fastboot% flash %~1 %~2"

	SETLOCAL

	SET /A okay_count=0
	SET /A send_write_count=0

	FOR /F "usebackq tokens=*" %%X in (`%fastboot% flash %~1 %~2 2^>^&1`) do (
		ECHO. %%X

		FOR /F "usebackq tokens=1" %%C in (`echo %%X ^| find /C "OKAY"`) do (
			IF "%%C" == "1" SET /A okay_count+=1
		)

		FOR /F "usebackq tokens=1" %%C in (`echo %%X ^| find /C "sending '%~1'"`) do (
			IF "%%C" == "1" SET /A send_write_count+=1
		)

		FOR /F "usebackq tokens=1" %%C in (`echo %%X ^| find /C "writing '%~1'"`) do (
			IF "%%C" == "1" SET /A send_write_count+=1
		)
	)

	IF %okay_count% LSS 2 (
		ECHO. %fastboot% flash %~1 %~2 failed!!
		ENDLOCAL
		EXIT /b 1
	)

	IF %okay_count% NEQ %send_write_count% (
		ECHO. %fastboot% flash %~1 %~2 failed!!
		ENDLOCAL
		EXIT /b 1
	)

	ECHO.

	ENDLOCAL
	EXIT /b 0

:fastboot_multi_flash
	ECHO. Executing "%fastboot% flash %~1 %~2"

	SETLOCAL

	SET /A okay_count=0
	SET /A send_write_count=0

	FOR /F "usebackq tokens=*" %%X in (`%fastboot% flash %~1 %~2 2^>^&1`) do (
		ECHO. %%X

		FOR /F "usebackq tokens=1" %%C in (`echo %%X ^| find /C "OKAY"`) do (
			IF "%%C" == "1" SET /A okay_count+=1
		)

		FOR /F "usebackq tokens=1" %%C in (`echo %%X ^| find /C "sending '%~1'"`) do (
			IF "%%C" == "1" SET /A send_write_count+=1
		)

		FOR /F "usebackq tokens=1" %%C in (`echo %%X ^| find /C "writing '%~1'"`) do (
			IF "%%C" == "1" SET /A send_write_count+=1
		)
	)

	IF %okay_count% LSS 2 (
		ECHO. %fastboot% flash %~1 %~2 failed!!
		ENDLOCAL
		EXIT /b 1
	)

	IF %okay_count% LSS %send_write_count% (
		ECHO. %fastboot% flash %~1 %~2 failed!!
		ENDLOCAL
		EXIT /b 1
	)

	ECHO.

	ENDLOCAL
	EXIT /b 0

:fastboot_reboot_bl
	ECHO. Executing %fastboot% reboot-bootloader

	SETLOCAL

	SET /A okay_count=0

	FOR /F "usebackq tokens=*" %%X in (`%fastboot% reboot-bootloader 2^>^&1`) do (
		ECHO. %%X

		FOR /F "usebackq tokens=1" %%C in (`echo %%X ^| find /C "OKAY"`) do (
			IF "%%C" == "1" SET /A okay_count+=1
		)
	)

	IF %okay_count% NEQ 1 (
		ECHO. %fastboot% reboot-bootloader failed!!
		PAUSE
		ENDLOCAL
		EXIT /b 1
	)

	: to allow time for fastboot de-enumeration before continuing
	ping -n 5 127.0.0.1 > NUL

	ENDLOCAL
	EXIT /b 0

:fastboot_erase
        ECHO. Executing "%fastboot% erase %~1"

        SETLOCAL

        SET /A okay_count=0

        FOR /F "usebackq tokens=*" %%X in (`%fastboot% erase %~1  2^>^&1`) do (
                ECHO. %%X

                FOR /F "usebackq tokens=1" %%C in (`echo %%X ^| find /C "OKAY"`) do (
                        IF "%%C" == "1" SET /A okay_count+=1
                )
        )

        IF %okay_count% NEQ 1 (
                ECHO. %fastboot% erase %~1  failed!!
                PAUSE
                ENDLOCAL
                EXIT /b 1
        )

        ECHO.

        ENDLOCAL
        EXIT /b 0
